using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

/* editor control for pic editor form */

namespace Uploader
{
    public partial class EditorCtrl : Control
    {

        public enum EditMode { Select, FillRect, Circle, Line, Pencil };
        public EditMode Mode = EditMode.Select;

        public Color EditColour = Color.Black;
        public int LineWidth = 1;

        private Image undoImage = null;
        private Image bufferImage = null;
        private Image resizedImage = null;
        private Image image = null;

        private Point startPoint = new Point();
        private Point immPoint = new Point();

        private Pen selectionPen;
   
        private List<Point> points = new List<Point>();

        private bool hasResized = true;

        private int xO, yO; // old image res
        private int xS, yS; // x, y offsets for centering

        System.Threading.Timer ants;

        public Image Image
        {
            get
            {
                return image;
            }
            set
            {
                image = value;
                Reset();
            }
        }

        public bool HasSelectedRegion
        {
            get
            {
                return (immPoint != startPoint);
            }
            set
            {
                if (!value) // if setting to false         
                    startPoint = immPoint;
                
            }
        }

        private float PicScale
        {
            get
            {
                return ((float)Image.Width / (float)xO);
            }
        }

        public EditorCtrl()
        {

            //AllocConsole();

            InitializeComponent();

            SetStyle(ControlStyles.UserPaint, true);
            SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            SetStyle(ControlStyles.DoubleBuffer, true);

            selectionPen = new Pen(Brushes.GreenYellow,3F);
            selectionPen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dash;

            ants = new System.Threading.Timer(new System.Threading.TimerCallback(AntsProc));
            ants.Change(100,100);

            AllowDrop = true;

            DragDrop += new DragEventHandler(DDEvent);
            DragEnter += new DragEventHandler(DDEnter);
        }

        private void DDEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false))
                e.Effect = DragDropEffects.All;

        }

        private void DDEvent(object sender, DragEventArgs e)
        {

            string fileName = ((string[])e.Data.GetData(DataFormats.FileDrop))[0];

            try
            {    // prevent file-locked stuff
                 Image = Funcs.CleanseImage(new Bitmap(fileName)); // fails if the file is not a valid image

                 try // this will fail if the control is used elsewhere (not a piceditorform)
                 {
                     fileName = fileName.Substring(fileName.LastIndexOf("\\") + 1);
                     fileName = fileName.Substring(0, fileName.LastIndexOf("."));
                     (this.Parent as EditorForm).Filename = fileName;
                 }
                 catch { }
            }
            catch
            {
                MessageBox.Show(fileName + " is not a valid image format.", "Error opening file", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }


        }

        public void AntsProc(object n)
        {
            if (!HasSelectedRegion || (Mode != EditMode.Select))
                return;

            try
            {
                selectionPen.DashOffset++;

                if (selectionPen.DashOffset > 10)
                    selectionPen.DashOffset = 0;

                Refresh();
            }
            catch { }
        }

        public void CropToSelection()
        {

            if (!HasSelectedRegion) // no selection
                return;

            if (Image != null)     
                lock (Image)
                {
                    try
                    {
                        Image = (Image as Bitmap).Clone(SelectionToPicRect(), Image.PixelFormat);
                    }
                    catch { }
                }         

        }

        public void Reset()
        {
            HasSelectedRegion = false;

            xO = -1; // cause a resize
            yO = -1;

            Refresh();
        }

        protected override void OnPaint(PaintEventArgs pe)
        {
            if (hasResized) // redo the doublebuffer image
            {
                hasResized = false;

                if (bufferImage != null)
                {
                    bufferImage.Dispose();
                    bufferImage = null;
                }

                bufferImage = new Bitmap(Width, Height, System.Drawing.Imaging.PixelFormat.Format24bppRgb);
                Reset();
            }

            Graphics g = Graphics.FromImage(bufferImage);
            g.Clear(Color.White);

            if (Image != null)
            {
                lock (Image)
                {
                    int w = (Height * Image.Width / Image.Height);
                    int h = (w * Image.Height / Image.Width);

                    if (w > Width)
                    {
                        h = (Width * Image.Height / Image.Width);
                        w = (h * Image.Width / Image.Height);
                    }

                    xS = 0;
                    yS = 0;

                    if (w < Width)
                        xS = (Width - w) / 2;

                    if (h < Height)
                        yS = (Height - h) / 2;

                    if ((resizedImage == null) || (w != xO) || (h != yO))
                    {
                        if (resizedImage != null)
                        {
                            resizedImage.Dispose();
                            resizedImage = null;
                        }

                        resizedImage = new Bitmap(w, h);

                        Graphics x = Graphics.FromImage(resizedImage);
                        x.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.Bicubic;

                        x.DrawImage(Image, 0, 0, w, h);
                        x.Dispose();
                    }


                    xO = w;
                    yO = h;

                    g.DrawImageUnscaled(resizedImage, xS, yS);

                }
            }
            else
                g.DrawString("No picture loaded. \r\n\r\nSelect open from the file menu and browse for a file, or drag and drop a file here.", this.Font, Brushes.Black, 1, 1);

            rectCheck();

            if (HasSelectedRegion)
            {
                g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighSpeed;

                switch (Mode)
                {
                    case EditMode.Select:
                        g.DrawRectangle(selectionPen, SelectionRect());
                        break;
                    case EditMode.FillRect:
                        g.FillRectangle(new SolidBrush(EditColour), SelectionRect());
                        break;
                    case EditMode.Circle:
                        g.DrawEllipse(new Pen(EditColour, LineWidth/PicScale), SelectionRect());
                        break;
                    case EditMode.Line:
                        g.DrawLine(new Pen(EditColour, LineWidth / PicScale), startPoint, immPoint);
                        break;
                    case EditMode.Pencil:

                        // rounded ends
                        Point pt = points[0];
                        pt.X = (int)(pt.X - LineWidth / PicScale / 2);
                        pt.Y = (int)(pt.Y - LineWidth / PicScale / 2);

                        g.FillEllipse(new SolidBrush(EditColour), new Rectangle(pt, new Size((int)(LineWidth / PicScale), (int)(LineWidth / PicScale))));
                        
                        pt = points[points.Count - 1];
                        pt.X = (int)(pt.X - LineWidth / PicScale / 2);
                        pt.Y = (int)(pt.Y - LineWidth / PicScale / 2); 
                        
                        g.FillEllipse(new SolidBrush(EditColour), new Rectangle(pt, new Size((int)(LineWidth / PicScale), (int)(LineWidth / PicScale))));                        

                        if (points.Count > 2)    // draw the line                   
                            g.DrawLines(new Pen(EditColour, LineWidth / PicScale), points.ToArray());

                        break;
                }
            }

            g.DrawRectangle(Pens.Black, 0, 0, Width - 1, Height - 1);
            g.Dispose();

            pe.Graphics.DrawImageUnscaled(bufferImage, 0, 0);        

        }

        protected override void OnResize(EventArgs e)
        {
            hasResized = true;
            ants.Change(100, 100);
        }

        // causes flicker.
        protected override void OnPaintBackground(PaintEventArgs pevent) { }

        protected override void OnVisibleChanged(EventArgs e)
        {
            if (this.Visible)
                this.Refresh();
        }

        [System.Runtime.InteropServices.DllImport("kernel32")]
        static extern bool AllocConsole();
        int count = 0;
        protected override void OnMouseDown(MouseEventArgs e)
        {
            if (Image != null)
            {
                if (e.Button == MouseButtons.Left)
                {
                    
                    startPoint = e.Location;
                    immPoint = e.Location;
                    rectCheck();

                    points.Clear();
                    points.Add(startPoint);

                    Refresh();
                }
                else if (e.Button == MouseButtons.Right)
                    Reset();
            }
        }

        protected override void OnMouseMove(MouseEventArgs e)
        {
            if ((Image != null) && (e.Button == MouseButtons.Left))
            {

                if (Mode == EditMode.Pencil)
                {
                    rectCheck();
                    points.Add(immPoint);
                }

                immPoint = e.Location;
                Refresh();

            }

        }

        protected override void OnMouseUp(MouseEventArgs e)
        {
            if (Image != null)
            {
                Graphics g;

                try
                {
                    g = Graphics.FromImage(Image);
                }
                catch // happens when trying to draw on an image that was a gif
                {
                    Bitmap tmp = new Bitmap(Image.Width, Image.Height);

                    g = Graphics.FromImage(tmp);
                    g.DrawImageUnscaled(Image, 0, 0);
                }

                if (undoImage != null)
                    undoImage.Dispose();

                undoImage = (Image)Image.Clone();

                // Tihi, pretty.
                g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
                
                switch (Mode) // Apply the drawn shape
                {
                    case EditMode.FillRect:
                        g.FillRectangle(new SolidBrush(EditColour), SelectionToPicRect());
                        break;
                    case EditMode.Circle:
                        g.DrawEllipse(new Pen(EditColour, LineWidth), SelectionToPicRect());
                        //Console.WriteLine("g.DrawEllipse(new Pen(Colors.Black, " + LineWidth + "), " + SelectionToPicRect() + "); // " + count);
                        count++;
                        
                        break;
                    case EditMode.Line:

                        g.DrawLine(new Pen(EditColour, LineWidth), PointToPic(startPoint), PointToPic(immPoint));
                        //Console.WriteLine("g.DrawLine(new Pen(Colors.Black, " + LineWidth + "), " + PointToPic(startPoint).X + ", " + PointToPic(startPoint).Y + ", " + PointToPic(immPoint).X + ", " + PointToPic(immPoint).Y + "); // " + count);
                        count++;
                        break;
                    case EditMode.Pencil:

                        if (points.Count > 0)
                        {

                            Point[] p = new Point[points.Count];
                            int i = 0;

                            foreach (Point s in points)
                            {
                                p[i] = PointToPic(s);
                                i++;
                            }

                            // nice rounded ends
                            Point pt = PointToPic(points[0]);
                            pt.X = pt.X - LineWidth / 2;
                            pt.Y = pt.Y - LineWidth / 2;

                            g.FillEllipse(new SolidBrush(EditColour), new Rectangle(pt, new Size(LineWidth, LineWidth)));

                            pt = PointToPic(points[points.Count - 1]);
                            pt.X = pt.X - LineWidth / 2;
                            pt.Y = pt.Y - LineWidth / 2;

                            g.FillEllipse(new SolidBrush(EditColour), new Rectangle(pt, new Size(LineWidth, LineWidth)));                        

                            if (points.Count > 2)                           
                                g.DrawLines(new Pen(EditColour, LineWidth), p);
                                             
                            
                        }

                        break;

                } 
                g.Dispose();

                if (Mode != EditMode.Select)
                    Reset();
            }
                 
        }

        private void rectCheck() // bounds checking, can't select/draw outside picture
        {
            if (startPoint.X < xS)
                startPoint.X = xS;

            if (startPoint.Y < yS)
                startPoint.Y = yS;

            if (startPoint.X > (xS + xO))
                startPoint.X = (xS + xO) - 1;

            if (startPoint.Y > (yS + yO))
                startPoint.Y = (yS + yO) - 1;

            if (immPoint.X < xS)
                immPoint.X = xS;

            if (immPoint.Y < yS)
                immPoint.Y = yS;

            if (immPoint.X > (xS + xO))
                immPoint.X = (xS + xO) - 1;

            if (immPoint.Y > (yS + yO))
                immPoint.Y = (yS + yO) - 1;
        }

        public void doDispose() // clean everything up... (me being paranoid)
        {
            if (resizedImage != null)
                resizedImage.Dispose();

            if (Image != null) 
                Image.Dispose();

            if (bufferImage != null)
                bufferImage.Dispose();

            if (undoImage != null)
                undoImage.Dispose();

            selectionPen.Dispose();

        }

        private Rectangle SelectionToPicRect() // get the location framed by the two points, if were to draw to it.
        {

            int startPointX = (int)((startPoint.X - xS) * PicScale);
            int startPointY = (int)((startPoint.Y - yS) * PicScale);

            int immPointX = (int)((immPoint.X - xS) * PicScale);
            int immPointY = (int)((immPoint.Y - yS) * PicScale);

            if (startPointX > immPointX)
            {
                int i = immPointX;
                immPointX = startPointX;
                startPointX = i;
            }

            if (startPointY > immPointY)
            { // there's gotta be a better way to do this.
                int i = immPointY;
                immPointY = startPointY;
                startPointY = i;
            }

            return new Rectangle(startPointX, startPointY, Math.Abs(startPointX - immPointX) + 1, Math.Abs(startPointY - immPointY) + 1);

        }

        private Point PointToPic(Point p) // change a client point into a picture-scaled point
        {
            int PointX = (int)((p.X - xS) * PicScale);
            int PointY = (int)((p.Y - yS) * PicScale);

            return new Point(PointX, PointY);
        }


        private Rectangle SelectionRect()
        { // get the selected rect to be used for preview

            int startPointX = (startPoint.X);
            int startPointY = (startPoint.Y);

            int immPointX = (immPoint.X);
            int immPointY = (immPoint.Y);

            if (startPointX > immPointX)
            {
                int i = immPointX;
                immPointX = startPointX;
                startPointX = i;
            }

            if (startPointY > immPointY)
            {
                int i = immPointY;
                immPointY = startPointY;
                startPointY = i;
            }

            return new Rectangle(startPointX, startPointY, Math.Abs(startPointX - immPointX) + 1, Math.Abs(startPointY - immPointY) + 1);

        }

        public void Undo()
        {
            Image i = Image;
            Image = undoImage;
            undoImage = i;
        }

        public void DoResize(int w, int h)
        {
            if (Image != null)
                lock (Image)
                {
                    try
                    {
                        Bitmap b = new Bitmap(w, h);
                        Graphics x = Graphics.FromImage(b);
                        x.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.Bicubic;

                        x.DrawImage(Image, 0, 0, w, h);
                        x.Dispose();

                        Image.Dispose();
                        Image = b;
                    }
                    catch { }
                }     
        }
    }
}
