        using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;

namespace Uploader
{
    class WinampNP {

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        private static extern IntPtr SendMessage(IntPtr hWnd, UInt32 Msg, int wParam, int lParam);

        [DllImport("user32.dll", SetLastError = true)]
        static extern IntPtr FindWindow(string lpClassName, string lpWindowName);

        [DllImport("user32.dll", SetLastError = true)]
        static extern uint GetWindowThreadProcessId(IntPtr hWnd, out uint lpdwProcessId);

        [DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        static extern uint RegisterWindowMessage(string lpString);

        [DllImport("kernel32.dll")]
        public static extern IntPtr OpenProcess(int dwDesiredAccess, bool bInheritHandle, uint dwProcessId);

        [DllImport("kernel32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool CloseHandle(IntPtr hObject);

        [DllImport("kernel32.dll", SetLastError = true)]
        static extern bool ReadProcessMemory(
          IntPtr hProcess,
          IntPtr lpBaseAddress,
          [Out] byte[] lpBuffer,
          int dwSize,
          out int lpNumberOfBytesRead
        );

        private static string ReadWinampString(IntPtr hwnd, IntPtr location) {
            uint pid;
            IntPtr process;

            GetWindowThreadProcessId(hwnd, out pid);
            if (pid == 0)
                return string.Empty;

            if ((process = OpenProcess(0x0008 | 0x0010, false, pid)) == IntPtr.Zero)
                return string.Empty;

            int bytesRead;
            byte[] buffer = new byte[1024];

            try {
                if (!ReadProcessMemory(process, location, buffer, buffer.Length, out bytesRead))
                    return string.Empty;
            } finally {
                CloseHandle(process);
            }

            string str = Encoding.GetEncoding(1252 /* windows-1252 - windows extended ASCII */).GetString(buffer);
            return str.Contains("\0") ? str.Substring(0, str.IndexOf('\0')) : str;
        }

        public static string GetFilename() {
            IntPtr hwnd;
            if ((hwnd = GetWinampHandle()) == null)
                return "";
            int plPos = (int)SendMessage(hwnd, 0x0400 /* WM_USER/WM_WA_IPC */, 0, 125 /*IPC_GETLISTPOS*/);

            return ReadWinampString(hwnd, SendMessage(hwnd, 0x0400 /* WM_USER/WM_WA_IPC */, plPos, 211 /*IPC_GETPLAYLISTFILE*/));
        }

        private static IntPtr GetWinampHandle() {
            return FindWindow("Winamp v1.x", null);
        }
    }
     
    class Funcs
    {
        private static string[] prefixes = { "b", "kb", "mb", "gb", "tb" };
        public static string GetNiceByteStr(long bytes) // convert bytes into a readable amount
        {
            int pr = 0;
            double nr = bytes;

            while (nr > 1024)
            {
                nr = nr / 1024;
                pr++;
            }

            return Math.Round(nr, (pr > 2) ? 2 : 1) + prefixes[pr].ToUpper();
        }

        public static void AddNewMenuItem(MenuItem menu, string Text, EventHandler handler)
        {
            AddNewMenuItem(menu, Text, handler, Shortcut.None);
        }

        public static void AddNewMenuItem(MenuItem menu, string Text, EventHandler handler, Shortcut s)
        {
            MenuItem i = new MenuItem(Text, handler);
            if (s != Shortcut.None)
            {
                i.Shortcut = s;
                i.ShowShortcut = true;
            }

            menu.MenuItems.Add(i);
        }

        public static void AddSeperator(MenuItem m)
        { // add vertical seperator to a menu
            m.MenuItems.Add(new MenuItem("-"));
        }

        public static string GetNiceTimeStr(long seconds)
        { // seconds -> time guess
            string label = " second";

            if (seconds > 3600)
            {
                seconds = (seconds / 3600);
                label = " hour";
            }
            else if (seconds > 60)
            {
                seconds = (seconds / 60);
                label = " minute";
            }

            if (seconds != 1)
                label += "s";
            
            return seconds + label;
        }

        public static string RandomAlphanumeric()
        { // gets a 32 char long mix of letters and numbers some uppercase

            StringBuilder randName = new StringBuilder();
            Random generator = new Random();

            for (int i = 0; i < 32; i++)
            {
                int c = generator.Next(36);
                if (c > 9)
                {
                    string s = ((char)(c + 55)).ToString();
                    randName.Append(((generator.Next(2) - 1) == 0) ? s : s.ToLower());
                }
                else randName.Append(c.ToString());

            }

            return randName.ToString();
        }

        internal static System.Drawing.Image CleanseImage(System.Drawing.Image Image)
        { // create a copy of an image so the source (file on disk) is not locked.
            System.Drawing.Bitmap tmp = new System.Drawing.Bitmap(Image.Width, Image.Height);

            System.Drawing.Graphics g = System.Drawing.Graphics.FromImage(tmp);
            g.DrawImageUnscaled(Image, 0, 0);
            g.Dispose();
            Image.Dispose();

            return tmp;

        }

        public static string ROT13P(string str)
        { // ROT13's alphanumeric chars and in addition rotates numbers too

            if (str == null)
                return null;

            StringBuilder sb = new StringBuilder(str.Length);

            foreach (char chr in str) {
                int c = (int)chr;

                if (Char.IsLetter(chr))
                {               
                    c += 13;

                    if (((c > 90) && Char.IsUpper(chr)) || (c > 122))
                        c -= 26;

                }
                else if (Char.IsNumber(chr))
                {
                    c += 5;

                    if (c > 57)
                        c -= 10;
                 
                }

                sb.Append((char)c);
            }

            return sb.ToString();
        }

    }
}
