using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Net;
using System.Threading;

namespace Uploader {
    public partial class UploadForm : Form {

        public bool Ready {
            get {
                return (buttGo.Text == "Go");
            }
        }

        public string Filename // these are a nice way to expose private fields such as that of a control to the outside
        {
            get {
                return txtFile.Text;
            }
            set {
                txtFile.Text = value;
            }
        }

        public bool OperationSuccess = false;
        public bool Stopping = false;
        public bool TerminateOnCompletion = false;

        public UploadForm() {
            InitializeComponent(); // Designer init. Don't remove

            lblStatus.Text = "Select a file";
            lblPerSec.Text = "";
            lblPercent.Text = "";
            lblGuess.Text = "";

            // confiure stuff for drag and drops
            AllowDrop = true;
            DragDrop += new DragEventHandler(DDEvent);
            DragEnter += new DragEventHandler(DDEnter);

            foreach (Control c in this.Controls) {
                c.AllowDrop = true;
                c.DragDrop += new DragEventHandler(DDEvent);
                c.DragEnter += new DragEventHandler(DDEnter);
            }

            // automatic copy to clipboard
            txtAddr.MouseClick += new MouseEventHandler(addrClick);
            txtAddr.MouseDoubleClick += new MouseEventHandler(addrClick);

            toolTip.SetToolTip(buttBrowse, "Browse for a file to upload");
            toolTip.SetToolTip(buttCP, "Upload a picture");
            toolTip.SetToolTip(buttGo, "Start/Stop upload");
            toolTip.SetToolTip(buttSetup, "Configure server info");

        }

        void addrClick(object sender, MouseEventArgs e) { // put to clipboard
            if (txtAddr.Text != "") {
                txtAddr.SelectAll();
                Clipboard.SetDataObject(txtAddr.Text, true, 10, 50);
            }
        }

        protected override void OnShown(EventArgs e) { // happens when the form is shown for the first time, and only then.
            base.OnShown(e);

            if (Filename != "") // command-line argument; execute and close.
            {
                TerminateOnCompletion = true;
                Execute();
            } else {

                try {

                    string maybename = Clipboard.GetText();
                    if (File.Exists(maybename)) {
                        Filename = maybename;
                    }
                } catch { }
            }

        }

        private void DDEnter(object sender, DragEventArgs e) { // occurs when mouse enters control area with a drag and drop item
            // effect is set if the control can accept the data
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false) && Ready)
                e.Effect = DragDropEffects.Copy;

        }

        private void DDEvent(object sender, DragEventArgs e) { // item has been dropped

            if (Ready) {
                Filename = ((string[])e.Data.GetData(DataFormats.FileDrop))[0];
                Execute();
            }
        }

        private void DoUpload() // execute the upload.
        {

            buttBrowse.Enabled = false;
            buttSetup.Enabled = false;
            buttCP.Enabled = false;
            button1.Enabled = false;

            OperationSuccess = false;
            progBar.Value = 0;

            lblStatus.Text = "Connecting";

            FileInfo fileInf = new FileInfo(Filename);

            string uri = Config.FtpPath + fileInf.Name;

            if (!uri.StartsWith("ftp://")) // it'll try to do wrong things otherwise.
            {
                lblPercent.Text = "";
                lblStatus.Text = "Failure: check configuration";

                MessageBox.Show("Server does not begin with ftp://", "Upload Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            } else {
                int bytesUploaded = 0; // last chunk size

                long totalUploaded = 0; // total uploaded
                long lastUlCount = 0; // last uploaded count (bytes)
                long fileLength = fileInf.Length;


                FtpWebRequest reqFTP = (FtpWebRequest)FtpWebRequest.Create(new Uri(uri));

                reqFTP.Method = WebRequestMethods.Ftp.UploadFile;
                reqFTP.Credentials = new NetworkCredential(Config.UserName, Config.Password);

                reqFTP.KeepAlive = false;
                reqFTP.UseBinary = true;
                reqFTP.Proxy = null; // will fail when using a http proxy, so turn it off.

                reqFTP.ContentLength = fileLength;

                lblStatus.Text = "Connected";

                TimeSpan ts;
                DateTime last = DateTime.UtcNow;

                byte[] buff = new byte[1024];

                try {
                    lblStatus.Text = "Starting";

                    FileStream fs = fileInf.OpenRead();
                    Stream strm = reqFTP.GetRequestStream();

                    txtAddr.Text = Config.WebPath + System.Web.HttpUtility.UrlPathEncode(fileInf.Name);

                    string len = Funcs.GetNiceByteStr(fileLength);

                    string addr = txtAddr.Text + " (" + len + ")";

                    //Clipboard.Clear();
                    try {
                        Thread.Sleep(200);
                        Clipboard.SetDataObject(addr, true, 10, 50);
                    } catch {
                        try {
                            Thread.Sleep(200);
                            Clipboard.SetDataObject(addr, true, 10, 100);
                        } catch {
                            try {
                                Thread.Sleep(200);
                                Clipboard.SetDataObject(addr, true, 10, 200);
                            } catch {
                                MessageBox.Show("Fuck you clipboard");
                            }
                        }
                    }

                    int updTime = 3;

                    do {

                        bytesUploaded = fs.Read(buff, 0, buff.Length);
                        totalUploaded += bytesUploaded;

                        strm.Write(buff, 0, bytesUploaded);

                        progBar.Value = Math.Abs((int)(totalUploaded * 100 / fileLength));

                        lblStatus.Text = Funcs.GetNiceByteStr(totalUploaded) + " / " + len;
                        lblPercent.Text = progBar.Value + "%";

                        ts = DateTime.UtcNow - last; // bytes per second indication
                        if (ts.TotalSeconds >= 1) {
                            last = DateTime.UtcNow;

                            int lastTransfered = (int)(totalUploaded - lastUlCount);
                            lblPerSec.Text = Funcs.GetNiceByteStr(lastTransfered) + "/s";
                            int guessTime = (int)((fileLength - totalUploaded) / lastTransfered);
                            string guess = Funcs.GetNiceTimeStr(guessTime);
                            lblGuess.Text = guess + " remaining";

                            lastUlCount = totalUploaded;


                            if (updTime++ == 5) {
                               // updTime = 0;
                                try {
                                    int t = (int)Math.Ceiling((double)guessTime / 60);
                                    Clipboard.SetDataObject("WAIT " + t + " minute" + (t != 1 ? "s" : "") + ": " + addr, true, 10, 200);
                                } catch {
                                    
                                }
                            }
                        }

                    } while ((bytesUploaded != 0) && !Stopping); // will be zero when end of stream reached.

                    strm.Close(); // leak memory otherwise
                    fs.Close();

                    System.Media.SystemSounds.Exclamation.Play(); // ding ding!

                    this.TopMost = true; // cause the form to come to top briefly, as bringtofront does not do anything
                    this.TopMost = false;

                    lblPerSec.Text = "";

                    if (Stopping) // upload canceled.
                    {
                        lblPercent.Text = "";
                        lblStatus.Text = "Stopped";

                        TerminateOnCompletion = false;
                    } else {
                        lblStatus.Text = "Success!";
                        OperationSuccess = true;
                        lblPerSec.Text = Funcs.GetNiceByteStr(fileLength);


                        try {
                            string s = Clipboard.GetText();
                            if (s.Contains(addr)) {
                                Clipboard.SetText(addr);
                            }
                        } catch {
                        }

                        if (TerminateOnCompletion)
                            this.Close(); // close the form
                    }

                } catch (Exception ex) // server errored
                {
                    lblPercent.Text = "";
                    lblStatus.Text = "Failure...";

                    // need full newline sequence for win32 message boxes
                    MessageBox.Show("Upload failed!\r\n\r\n" + ex.Message, "Upload Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

            lblGuess.Text = "";
            buttGo.Text = "Go";

            buttCP.Enabled = true;
            buttBrowse.Enabled = true;
            buttSetup.Enabled = true;
            button1.Enabled = true;

        }

        private void goStart(object sender, EventArgs e) {
            Execute();
        }

        public void Execute() { // start/stop async upload.

            txtAddr.Text = "";
            lblPerSec.Text = "";
            lblPercent.Text = "";

            progBar.Value = 0;

            if (buttGo.Text == "Stop") {
                buttGo.Text = "Go";
                Stopping = true; // tell it to stop

                TerminateOnCompletion = false;
            } else {

                if ((Filename == "") || (Filename == "Picture from clipboard...")) {
                    Filename = "";
                    lblStatus.Text = "Please select a file";
                    return;
                } else if (!File.Exists(Filename)) {
                    lblStatus.Text = "File not found";
                    return;
                }

                buttGo.Text = "Stop";
                Stopping = false;

                Thread worker = new Thread(new ThreadStart(DoUpload));

                worker.IsBackground = true; // automatic termination when app closing
                worker.SetApartmentState(ApartmentState.STA); // needed for win32 calls (clipboard)
                worker.Start();
            }
        }

        private void buttBrowse_Click(object sender, EventArgs e) {
            lblStatus.Text = "Select a file";

            DialogResult r = openFileDialog.ShowDialog(); // show modal dialog

            if (r == DialogResult.OK) // user clicked okay
            {
                Filename = openFileDialog.FileName;
                lblStatus.Text = "Ready!";
            } else
                lblStatus.Text = "";

        }

        private void buttSetup_Click(object sender, EventArgs e) {
            new ConfigForm().ShowDialog();
        }

        private void buttCP_Click(object sender, EventArgs e) {
            new EditorForm(this).ShowDialog(); // this is so the piceditor knows its parent.
        }

        private void button1_Click(object sender, EventArgs e) {
            try {
                string fn = WinampNP.GetFilename();
                if (!string.IsNullOrEmpty(fn))
                    Filename = fn;
            } catch { }
        
        }
    }
}